@@ -1,6 +1,6 @@
-(* ocamlbuild plugin for building Batteries.  
+(* ocamlbuild plugin for building Batteries.
  * Copyright (C) 2010 Michael Ekstrand
- * 
+ *
  * Portions (hopefully trivial) from build/myocamlbuild.ml and the
  * Gallium wiki. *)
 
@@ -10,6 +10,9 @@ let ocamlfind x = S[A"ocamlfind"; A x]
 
 let packs = String.concat "," ["camomile"; "num"; "str"]
 
+let mkconf = "build/mkconf.byte"
+let pa_llist = "src/syntax/pa_llist/pa_llist.cmo"
+
 let _ = dispatch begin function
   | Before_options ->
       (* Set up to use ocamlfind *)
@@ -34,41 +37,49 @@ let _ = dispatch begin function
 
       rule "process config file"
         ~prod:"%.ml"
-        ~deps:["%.mlp"; "VERSION"; "mkconf.byte"]
+        ~deps:["%.mlp"; "VERSION"; mkconf]
         begin fun env build ->
-          Cmd(S[A"ocamlrun"; P"mkconf.byte"; P(env "%.mlp"); P(env "%.ml")])
+          Cmd(S[A"ocamlrun"; P mkconf; P(env "%.mlp"); P(env "%.ml")])
         end;
-      
+
       rule "process meta file"
         ~prod:"META"
-        ~deps:["META.in"; "VERSION"; "mkconf.byte"]
+        ~deps:["META.in"; "VERSION"; mkconf]
         begin fun env build ->
-          Cmd(S[A"ocamlrun"; P"mkconf.byte"; P"META.in"; P"META"])
+          Cmd(S[A"ocamlrun"; P mkconf; P"META.in"; P"META"])
         end
 
   | After_rules ->
       (* When one links an OCaml program, one should use -linkpkg *)
       flag ["ocaml"; "link"; "program"] & A"-linkpkg";
 
+(* Causes build to fail on armel under ocaml 3.10.2
+      flag ["ocaml"; "native"; "compile"] & A"-annot"; *)
+      flag ["ocaml"; "compile"] & S[A"-warn-error"; A"A"];
+
       flag ["ocaml"; "compile"] & S[A"-package"; A packs];
       flag ["ocaml"; "ocamldep"] & S[A"-package"; A packs];
       flag ["ocaml"; "doc"] & S[A"-package"; A packs];
       flag ["ocaml"; "link"] & S[A"-package"; A packs];
       flag ["ocaml"; "infer_interface"] & S[A"-package"; A packs];
 
-      flag ["ocaml"; "infer_interface"; "pkg_oUnit"] & S[A"-package"; A"oUnit"];
-      flag ["ocaml"; "ocamldep"; "pkg_oUnit"] & S[A"-package"; A"oUnit"];
-      flag ["ocaml"; "compile"; "pkg_oUnit"] & S[A"-package"; A"oUnit"];
-      flag ["ocaml"; "link"; "pkg_oUnit"] & S[A"-package"; A"oUnit"];
+      List.iter
+        (fun pkg ->
+          flag ["ocaml"; "infer_interface"; "pkg_"^pkg] & S[A"-package"; A pkg];
+          flag ["ocaml"; "ocamldep"; "pkg_"^pkg] & S[A"-package"; A pkg];
+          flag ["ocaml"; "compile"; "pkg_"^pkg] & S[A"-package"; A pkg];
+          flag ["ocaml"; "link"; "pkg_"^pkg] & S[A"-package"; A pkg];
+          ())
+        ["oUnit"; "benchmark"];
 
-      (* DON'T USE TAG 'thread', USE 'threads' 
+      (* DON'T USE TAG 'thread', USE 'threads'
 	 for compatibility with ocamlbuild *)
       flag ["ocaml"; "compile"; "threads"] & A"-thread";
       flag ["ocaml"; "link"; "threads"] & A"-thread";
       flag ["ocaml"; "doc"; "threads"] & S[A"-I"; A "+threads"];
 
       flag ["ocaml"; "doc"] & S[A"-hide-warnings"; A"-sort"];
-      
+
       flag ["ocaml"; "compile"; "camlp4rf"] &
         S[A"-package"; A"camlp4.lib"; A"-pp"; A"camlp4rf"];
       flag ["ocaml"; "ocamldep"; "camlp4rf"] &
@@ -78,5 +89,44 @@ let _ = dispatch begin function
         S[A"-package"; A"camlp4.lib"; A"-pp"; A"camlp4of"];
       flag ["ocaml"; "ocamldep"; "camlp4of"] &
         S[A"-package"; A"camlp4.lib"; A"-pp"; A"camlp4of"];
+
+      flag ["ocaml"; "compile"; "syntax_camlp4o"] &
+        S[A"-syntax"; A"camlp4o"; A"-package"; A"camlp4"];
+      flag ["ocaml"; "ocamldep"; "syntax_camlp4o"] &
+        S[A"-syntax"; A"camlp4o"; A"-package"; A"camlp4"];
+
+      flag ["ocaml"; "compile";  "with_pa_llist"] &
+        S[A"-ppopt"; P pa_llist];
+      flag ["ocaml"; "ocamldep";  "with_pa_llist"] &
+        S[A"-ppopt"; P pa_llist];
+      dep ["ocaml"; "ocamldep"; "with_pa_llist"] [pa_llist];
+
+      ocaml_lib "qtest/test_mods";
+      ocaml_lib "src/batteries";
+
+      flag ["ocaml"; "link"; "linkall"] & S[A"-linkall"];
+(*
+      dep ["ocaml"; "link"; "include_tests"; "byte"] & 
+	[Pathname.mk "qtest/test_mods.cma"];
+      dep ["ocaml"; "link"; "include_tests"; "native"] & 
+	[Pathname.mk "qtest/test_mods.cmxa"]; *)
+
+      (* Some .mli files use INCLUDE "foo.mli" to avoid interface duplication;
+         
+         The problem is that the automatic dependency detector of
+         ocamlbuild doesn't detect the implicit dependency on the
+         included .mli, and doesn't copy it into _build before
+         preprocessing the including file.
+
+         Here, we add flags denoting explicit dependencies on the
+         included .mli. This solution comes from the following
+         explanation of Xavier Clerc:
+           http://caml.inria.fr/mantis/print_bug_page.php?bug_id=5162
+      *)
+      dep ["pset_mli"] [Pathname.concat "src" "batPSet.mli"];
+      dep ["pmap_mli"] [Pathname.concat "src" "batPMap.mli"];
+
+
+
   | _ -> ()
 end
