@@ -92,7 +92,7 @@ val iter : ('a -> unit) -> 'a t -> unit
 (** [iter f e] calls the function [f] with each elements of [e] in turn. *)
 
 val iter2 : ('a -> 'b -> unit) -> 'a t -> 'b t -> unit
-(** [iter2 f e1 e2] calls the function [f] with the next elements of [e] and
+(** [iter2 f e1 e2] calls the function [f] with the next elements of [e1] and
  [e2] repeatedly until one of the two enumerations ends. *)
 
 val exists: ('a -> bool) -> 'a t -> bool
@@ -106,23 +106,23 @@ val fold : ('b -> 'a -> 'b) -> 'b -> 'a t -> 'b
 (** A general loop on an enumeration.
 
     If [e] is empty, [fold f v e] returns [v]. Otherwise, [fold v e]
-    returns [f (... (f (f v a1) a2) ...) aN] where a1..N are the
+    returns [f (... (f (f v a0) a1) ...) aN] where [a0,a1..aN] are the
     elements of [e]. This function may be used, for instance, to
     compute the sum of all elements of an enumeration [e] as follows:
-    [fold ( + ) 0 e].
+    [fold ( + ) 0 e]. Eager.
 *)
 
 val reduce : ('a -> 'a -> 'a) -> 'a t -> 'a
-  (** A simplified version of [fold], which uses the first element
-      of the enumeration as a default value.
+(** A simplified version of [fold], which uses the first element
+    of the enumeration as a default value.
 
-      [fold f e] throws [Not_found] if [e] is empty, returns its only
-      element if e is a singleton, otherwise [f (... (f (f a1 a2)
-      a3)...) aN] where a1..N are the elements of [e]. *)
+      [reduce f e] throws [Not_found] if [e] is empty, returns its only
+      element if e is a singleton, otherwise [f (... (f (f a0 a1)
+      a2)...) aN] where [a0,a1..aN] are the elements of [e]. *)
 
 val sum : int t -> int
   (** [sum] returns the sum of the given int enum.  If the argument is
-      empty, returns 0 *)
+      empty, returns 0. Eager *)
 
 val fold2 : ('a -> 'b -> 'c -> 'c) -> 'c -> 'a t -> 'b t -> 'c
   (** [fold2] is similar to [fold] but will fold over two enumerations at the
@@ -130,21 +130,18 @@ val fold2 : ('a -> 'b -> 'c -> 'c) -> 'c -> 'a t -> 'b t -> 'c
 
 val scanl : ('b -> 'a -> 'b) -> 'b -> 'a t -> 'b t
 (** A variant of [fold] producing an enumeration of its intermediate values.
-    If [e] contains [x1], [x2], ..., [scanl f init e] is the enumeration 
-    containing [init], [f init x1], [f (f init x1) x2]... *)
+    If [e] contains [x0], [x1], ..., [scanl f init e] is the enumeration 
+    containing [init], [f init x0], [f (f init x0) x1]... Lazy. *)
 
 val scan : ('a -> 'a -> 'a) -> 'a t -> 'a t
 (** [scan] is similar to [scanl] but without the [init] value: if [e]
-    contains [x1], [x2], [x3] ..., [scan f e] is the enumeration containing
-    [x1], [f x1 x2], [f (f x1 x2) x3]... 
-
+    contains [x0], [x1], [x2] ..., [scan f e] is the enumeration containing
+    [x0], [f x0 x1], [f (f x0 x1) x2]...
 
     For instance, [scan ( * ) (1 -- 10)] will produce an enumeration 
     containing the successive values of the factorial function.*)
 
 
-
-
 (** Indexed functions : these functions are similar to previous ones
  except that they call the function with one additional argument which
  is an index starting at 0 and incremented after each call to the function. *)
@@ -165,11 +162,13 @@ val find : ('a -> bool) -> 'a t -> 'a
       found element, or, raises [Not_found] if no such element exists
       in the enumeration, consuming the whole enumeration in the search.
       
-      Since [find] consumes a prefix of the enumeration, it can be used several 
-      times on the same enumeration to find the next element. *)
+      Since [find] (eagerly) consumes a prefix of the enumeration, it
+      can be used several times on the same enumeration to find the
+      next element. *)
 
 val is_empty : 'a t -> bool
-  (** [is_empty e] returns true if [e] does not contains any element. *)
+(** [is_empty e] returns true if [e] does not contains any element.
+    Forces at most one element. *)
 
 val peek : 'a t -> 'a option
   (** [peek e] returns [None] if [e] is empty or [Some x] where [x] is
@@ -231,16 +230,30 @@ val break : ('a -> bool) -> 'a t -> 'a t * 'a t
   (** Negated span.
       [break test e] is equivalent to [span (fun x -> not (test x)) e] *)
 
-val group : ('a -> bool) -> 'a t -> 'a t t
-  (** [group test e] devides [e] into an enumeration of enumerations, where
-      each sub-enumeration is the longest continuous enumeration of elements whose [test]
-      results are the same. *)
+val group : ('a -> 'b) -> 'a t -> 'a t t
+(** [group test e] divides [e] into an enumeration of enumerations,
+    where each sub-enumeration is the longest continuous enumeration
+    of elements whose [test] results are the same. 
+
+    [Enum.group (x -> x mod 2) [1;2;4;1] = [[1];[2;4];[1]]]
+
+    [Enum.group (fun x -> x mod 3) [1;2;4;1] = [[1];[2];[4;1]]]
+
+    [Enum.group (fun s -> s.[0]) ["cat"; "canary"; "dog"; "dodo"; "ant"; "cow"] = [["cat"; "canary"];["dog";"dodo"];["ant"];["cow"]]]
+
+*)
+
+val group_by : ('a -> 'a -> bool) -> 'a t -> 'a t t
+(** [group_by eq e] divides [e] into an enumeration of enumerations,
+    where each sub-enumeration is the longest continuous enumeration
+    of elements that are equal, as judged by [eq]
+*)
 
 val clump : int -> ('a -> unit) -> (unit -> 'b) -> 'a t -> 'b t
-  (** [clump size add get e] runs [add] on [size] (or less at the end)
-      elements of [e] and then runs [get] to produce value for the
-      result enumeration.  Useful to convert a char enum into string
-      enum. *)
+(** [clump size add get e] runs [add] on [size] (or less at the end)
+    elements of [e] and then runs [get] to produce value for the
+    result enumeration.  Useful to convert a char enum into string
+    enum. *)
 
 (** {6 Lazy constructors}
 
@@ -252,28 +265,46 @@ val clump : int -> ('a -> unit) -> (unit -> 'b) -> 'a t -> 'b t
     underlying enumerations they were created from are also consumed. *)
 
 val map : ('a -> 'b) -> 'a t -> 'b t
-  (** [map f e] returns an enumeration over [(f a1, f a2, ... , f aN)] where
-      a1...N are the elements of [e]. *)
+  (** [map f e] returns an enumeration over [(f a0, f a1, ...)] where
+      [a0,a1...] are the elements of [e]. Lazy.
+   *)
   
 val mapi : (int -> 'a -> 'b) -> 'a t -> 'b t
-  (** [mapi] is similar to [map] except that [f] is passed one extra argument
-      which is the index of the element in the enumeration, starting from 0. *)
+(** [mapi] is similar to [map] except that [f] is passed one extra argument
+    which is the index of the element in the enumeration, starting from 0 :
+    mapi f e returns an enumeration over [(f 0 a0, f 1 a1, ...)] where
+    [a0,a1...] are the elements of [e]. *)
 
 val filter : ('a -> bool) -> 'a t -> 'a t
   (** [filter f e] returns an enumeration over all elements [x] of [e] such
-      as [f x] returns [true]. *)
+      as [f x] returns [true]. Lazy.
+     
+    {b Note} filter is lazy in that it returns a lazy enumeration, but
+    each element in the result is eagerly searched in the input
+    enumeration. Therefore, the access to a given element in the result
+    will diverge if it is preceded, in the input enumeration, by
+    infinitely many false elements (elements on which the predicate
+    [p] returns [false]).
+
+    Other functions that may drop an unbound number of elements
+    ([filter_map], [take_while], etc.) have the same behavior.  
+*)
 
 val filter_map : ('a -> 'b option) -> 'a t -> 'b t
-  (** [filter_map f e] returns an enumeration over all elements [x] such as
-      [f y] returns [Some x] , where [y] is an element of [e]. *)
+(** [filter_map f e] returns an enumeration over all elements [x] such as
+    [f y] returns [Some x] , where [y] is an element of [e].
+
+    [filter_map] works on infinite enumerations; see [filter].
+*)
   
 val append : 'a t -> 'a t -> 'a t
   (** [append e1 e2] returns an enumeration that will enumerate over all
-      elements of [e1] followed by all elements of [e2].
+      elements of [e1] followed by all elements of [e2]. Lazy.
       
       {b Note} The behavior of appending [e] to itself or to something
       derived from [e] is not specified. In particular, cloning [append e e]
-      may destroy any sharing between the first and the second argument.*)
+      may destroy any sharing between the first and the second argument.
+  *)
 
 val prefix_action : (unit -> unit) -> 'a t -> 'a t
 (** [prefix_action f e] will behave as [e] but guarantees that [f ()]
@@ -376,7 +407,7 @@ val seq : 'a -> ('a -> 'a) -> ('a -> bool) -> 'a t
 
 
 val unfold: 'b -> ('b -> ('a * 'b) option) -> 'a t
-  (**More powerful version of [seq], with the ability of hiding data.
+  (**As [from_loop], except uses option type to signal the end of the enumeration.
 
      [unfold data next] creates a (possibly infinite) enumeration from
      the successive results of applying [next] to [data], then to the
@@ -459,58 +490,12 @@ val hard_count : 'a t -> int
 (**
    {6 Utilities }
 *)
+
 val range : ?until:int -> int -> int t
 (** [range p until:q] creates an enumeration of integers [[p, p+1, ..., q]].
     If [until] is omitted, the enumeration is not bounded. Behaviour is 
     not-specified once [max_int] has been reached.*)
 
-val ( -- ) : int -> int -> int t
-(** As [range], without the label. 
-
-    [5 -- 10] is the enumeration 5,6,7,8,9,10.
-    [10 -- 5] is the empty enumeration*)
-
-val ( --^ ) : int -> int -> int t
-(** As [(--)] but without the right endpoint
-
-    [5 --^ 10] is the enumeration 5,6,7,8,9.
-*)
-
-val ( --. ) : (float * float) -> float -> float t
-(** [(a, step) --. b)] creates a float enumeration from [a] to [b] with an
-    increment of [step] between elements.
-
-    [(5.0, 1.0) --. 10.0] is the enumeration 5.0,6.0,7.0,8.0,9.0,10.0.
-    [(10.0, -1.0) --. 5.0] is the enumeration 10.0,9.0,8.0,7.0,6.0,5.0.
-    [(10.0, 1.0) --. 1.0] is the empty enumeration. *)
-
-val ( --- ) : int -> int -> int t
-(** As [--], but accepts enumerations in reverse order.
-
-    [5 --- 10] is the enumeration 5,6,7,8,9,10.
-    [10 --- 5] is the enumeration 10,9,8,7,6,5.*)
-
-val ( --~ ) : char -> char -> char t
-(** As ( -- ), but for characters.*)
-
-val ( // ) : 'a t -> ('a -> bool) -> 'a t
-(** Filtering (pronounce this operator name "such that").
-
-    For instance, [(1 -- 37) // odd] is the enumeration of all odd
-    numbers between 1 and 37.*)
-
-val ( /@ ) : 'a t -> ('a -> 'b) -> 'b t
-
-val ( @/ ) : ('a -> 'b) -> 'a t -> 'b t
-  (**
-     Mapping operators.
-
-     These operators have the same meaning as function {!map} but are
-     sometimes more readable than this function, when chaining
-     several transformations in a row.
-  *)
-
-
 val dup : 'a t -> 'a t * 'a t
   (** [dup stream] returns a pair of streams which are identical to [stream]. Note
       that stream is a destructive data structure, the point of [dup] is to
@@ -558,20 +543,118 @@ val switch : ('a -> bool) -> 'a t -> 'a t * 'a t
       all the elements satisfying [test], the second enum is opposite. The
       order of elements in the source enum is preserved. *)
 
+val partition : ('a -> bool) -> 'a t -> 'a t * 'a t
+  (** as [switch]
+
+      @added v1.4.0
+  *)
+
 (*val switchn: int -> ('a -> int) -> 'a t -> 'a t array
   (** [switchn] is the array version of [switch]. [switch n f fl] split [fl] to an array of [n] enums, [f] is
       applied to each element of [fl] to decide the id of its destination
       enum. *)*)
 
+val arg_min : ('a -> 'b) -> 'a t -> 'a
+val arg_max : ('a -> 'b) -> 'a t -> 'a
+(** [arg_min f xs] returns the [x] in [xs] for which [f x] is minimum.
+    Similarly for [arg_max], except it returns the maximum.  If
+    multiple values reach the maximum, one of them is
+    returned. (currently the first, but this is not guaranteed)
+
+    Example: [-5 -- 5 |> arg_min (fun x -> x * x + 6 * x - 5) = -3]
+    Example: [List.enum ["cat"; "canary"; "dog"; "dodo"; "ant"; "cow"] |> arg_max String.length = "canary"]
+
+    @added v1.4.0
+    @raises Invalid_argument if the input enum is empty
+*)
+
 (** {6 Trampolining} *)
 
 val while_do : ('a -> bool) -> ('a t -> 'a t) -> 'a t -> 'a t
 (** [while_do cont f e] is a loop on [e] using [f] as body and [cont] as
     condition for continuing. 
 
-    If [e] contains elements [x1], [x2], [x3]..., then if [cont x1] is [false], 
-    [x1] is returned as such and treatment stops. On the other hand, if [cont x1] 
-    is [true], [f x1] is returned and the loop proceeds with [x2]...*)
+    If [e] contains elements [x0], [x1], [x2]..., then if [cont x0] is [false], 
+    [x0] is returned as such and treatment stops. On the other hand, if [cont x0] 
+    is [true], [f x0] is returned and the loop proceeds with [x1]...
+
+    Note that f is used as halting condition {i after} the
+    corresponding element has been added to the result stream.
+*)
+
+(** {6 Infix operators} *)
+
+(** Infix versions of some functions
+
+	This module groups together all infix operators so that
+    you can open it without opening the whole batEnum module.
+*)
+
+module Infix : sig
+  val ( -- ) : int -> int -> int t
+(** As [range], without the label. 
+
+    [5 -- 10] is the enumeration 5,6,7,8,9,10.
+    [10 -- 5] is the empty enumeration*)
+
+  val ( --^ ) : int -> int -> int t
+(** As [(--)] but without the right endpoint
+
+    [5 --^ 10] is the enumeration 5,6,7,8,9.
+*)
+
+  val ( --. ) : (float * float) -> float -> float t
+(** [(a, step) --. b)] creates a float enumeration from [a] to [b] with an
+    increment of [step] between elements.
+
+    [(5.0, 1.0) --. 10.0] is the enumeration 5.0,6.0,7.0,8.0,9.0,10.0.
+    [(10.0, -1.0) --. 5.0] is the enumeration 10.0,9.0,8.0,7.0,6.0,5.0.
+    [(10.0, 1.0) --. 1.0] is the empty enumeration. *)
+
+  val ( --- ) : int -> int -> int t
+(** As [--], but accepts enumerations in reverse order.
+
+    [5 --- 10] is the enumeration 5,6,7,8,9,10.
+    [10 --- 5] is the enumeration 10,9,8,7,6,5.*)
+
+  val ( --~ ) : char -> char -> char t
+(** As ( -- ), but for characters.*)
+
+  val ( // ) : 'a t -> ('a -> bool) -> 'a t
+(** Filtering (pronounce this operator name "such that").
+
+    For instance, [(1 -- 37) // odd] is the enumeration of all odd
+    numbers between 1 and 37.*)
+
+  val ( /@ ) : 'a t -> ('a -> 'b) -> 'b t
+
+  val ( @/ ) : ('a -> 'b) -> 'a t -> 'b t
+  (**
+     Mapping operators.
+
+     These operators have the same meaning as function {!map} but are
+     sometimes more readable than this function, when chaining
+     several transformations in a row.
+  *)
+
+  val ( //@ ) : 'a t -> ('a -> 'b option) -> 'b t
+
+  val ( @// ) : ('a -> 'b option) -> 'a t -> 'b t
+  (**
+    Map combined with filter. Same as {!filter_map}.
+  *)
+end
+
+val ( -- ) : int -> int -> int t
+val ( --^ ) : int -> int -> int t
+val ( --. ) : (float * float) -> float -> float t
+val ( --- ) : int -> int -> int t
+val ( --~ ) : char -> char -> char t
+val ( // ) : 'a t -> ('a -> bool) -> 'a t
+val ( /@ ) : 'a t -> ('a -> 'b) -> 'b t
+val ( @/ ) : ('a -> 'b) -> 'a t -> 'b t
+val ( //@ ) : 'a t -> ('a -> 'b option) -> 'b t
+val ( @// ) : ('a -> 'b option) -> 'a t -> 'b t
 
 (** {6 Monad related modules} *)
 
@@ -584,12 +667,12 @@ module WithMonad : functor (Mon : BatMonad.S) -> sig
 (** Type of the monadic elements. *)
   
   val sequence : 'a m t -> 'a t m
-(** [sequence e] evaluates each monadic elements (of type ['a m] contained in the enumeration [e] to get a monadic enumeration of ['a] elements, 
-    of type ['a m BatEnum.t]. *)
+(** [sequence e] evaluates each monadic elements (of type ['a m]) contained in the enumeration [e] to get a monadic enumeration of ['a] elements, 
+    of type ['a BatEnum.t m]. *)
  
   val fold_monad : ('a -> 'b -> 'a m) -> 'a -> 'b t -> 'a m
 (** [fold_monad f init e] does a folding of the enumeration [e] applying step by step the function [f] that gives back results in the [Mon] monad, 
-    with the [init] initial element. The result is a value in the [Mond] monad. *)
+    with the [init] initial element. The result is a value in the [Mon] monad. *)
 end
 
 (** The BatEnum Monad 
