@@ -30,18 +30,19 @@
 
     {b Note} OCaml, Batteries Included, provides two implementations
     of sets: polymorphic sets and functorized sets. Functorized sets
-    (see {!S} and {!Make}) offer a more complex and slightly poorer
-    set of features but stronger type-safety. Polymorphic sets are
-    easier to use and have a few more powerful features but make it
-    easier to shoot yourself in the foot. In case of doubt,
-    functorized sets.
+    (see {!S} and {!Make}) are slightly more complex to use but offer
+    stronger type-safety. Polymorphic sets make it easier to shoot
+    yourself in the foot. In case of doubt, you should use functorized
+    sets.
 
     The functorized set implementation is built upon Stdlib's
     {{:http://caml.inria.fr/pub/docs/manual-ocaml/libref/Set.html}Set}
     module, but provides the complete interface.
 
-    @author Xavier Leroy (Base module)
-    @author David Teller
+    @author Xavier Leroy
+    @author Nicolas Cannasse
+    @author Markus Mottl
+    @author David Rajchenbach-Teller
 *)
 
 (** {4 Functorized Sets} *)
@@ -63,6 +64,9 @@ module type S =
     val is_empty: t -> bool
     (** Test whether a set is empty or not. *)
 
+    val singleton: elt -> t
+    (** [singleton x] returns the one-element set containing only [x]. *)
+
     val mem: elt -> t -> bool
     (** [mem x s] tests whether [x] belongs to the set [s]. *)
 
@@ -70,9 +74,6 @@ module type S =
     (** [add x s] returns a set containing all elements of [s],
        plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
 
-    val singleton: elt -> t
-    (** [singleton x] returns the one-element set containing only [x]. *)
-
     val remove: elt -> t -> t
     (** [remove x s] returns a set containing all elements of [s],
        except [x]. If [x] was not in [s], [s] is returned unchanged. *)
@@ -108,7 +109,7 @@ module type S =
 
     val map: (elt -> elt) -> t -> t
       (** [map f x] creates a new set with elements [f a0],
-	  [f a1]... [f an], where [a1], ..., [an] are the
+	  [f a1]... [f aN], where [a0],[a1]..[aN] are the
 	  values contained in [x]*)
 
     val filter: (elt -> bool) -> t -> t
@@ -117,14 +118,14 @@ module type S =
 
     val filter_map: (elt -> elt option) -> t -> t
       (** [filter_map f m] combines the features of [filter] and
-	  [map].  It calls calls [f a0], [f a1], [f an] where [a0..an]
+	  [map].  It calls calls [f a0], [f a1], [f aN] where [a0],[a1]..[aN]
 	  are the elements of [m] and returns the set of pairs [bi]
 	  such as [f ai = Some bi] (when [f] returns [None], the
 	  corresponding element of [m] is discarded). *)
 
     val fold: (elt -> 'a -> 'a) -> t -> 'a -> 'a
-      (** [fold f s a] computes [(f xN ... (f x2 (f x1 a))...)],
-	  where [x1 ... xN] are the elements of [s], in increasing order. *)
+      (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
+	  where [x0],[x1]..[xN] are the elements of [s], in increasing order. *)
 
     val for_all: (elt -> bool) -> t -> bool
     (** [for_all p s] checks if all elements of the set
@@ -140,6 +141,15 @@ module type S =
        predicate [p], and [s2] is the set of all the elements of
        [s] that do not satisfy [p]. *)
 
+    val split: elt -> t -> t * bool * t
+    (** [split x s] returns a triple [(l, present, r)], where
+          [l] is the set of elements of [s] that are
+          strictly less than [x];
+          [r] is the set of elements of [s] that are
+          strictly greater than [x];
+          [present] is [false] if [s] contains no element equal to [x],
+          or [true] if [s] contains an element equal to [x]. *)
+
     val cardinal: t -> int
     (** Return the number of elements of a set. *)
 
@@ -164,14 +174,9 @@ module type S =
        the set is empty. Which element is chosen is unspecified,
        but equal elements will be chosen for equal sets. *)
 
-    val split: elt -> t -> t * bool * t
-    (** [split x s] returns a triple [(l, present, r)], where
-          [l] is the set of elements of [s] that are
-          strictly less than [x];
-          [r] is the set of elements of [s] that are
-          strictly greater than [x];
-          [present] is [false] if [s] contains no element equal to [x],
-          or [true] if [s] contains an element equal to [x]. *)
+    val pop : t -> elt * t
+    (** returns one element of the set and the set without that element.
+        Raises [Not_found] if given an empty set *)
 
     val enum: t -> elt BatEnum.t
       (** Return an enumeration of all elements of the given set.
@@ -258,44 +263,30 @@ module CharSet    : S with type elt = Char.t
       depending on your situation *)
 
 module Make (Ord : OrderedType) : S with type elt = Ord.t
-  (** Functor building an implementation of the set structure
-      given a totally ordered type.
+(** Functor building an implementation of the set structure
+    given a totally ordered type.
+    
+    @documents Set.Make
+*)
 
-      @documents Set.Make
-  *)
+(** {6 Polymorphic sets}
 
-(*
- * PSet - Polymorphic sets
- * Copyright (C) 1996-2003 Xavier Leroy, Nicolas Cannasse, Markus Mottl
- * Copyright (C)      2009 David Rajchenbach-Teller, LIFO, Universite d'Orleans
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version,
- * with the special exception on linking described in file LICENSE.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *)
+   The definitions below describe the polymorphic set interface.
 
-(**
-   {4 Polymorphic sets}
+    They are similar in functionality to the functorized {!Make}
+    module, but the compiler cannot ensure that sets using different
+    element ordering have different types: the responsibility of not
+    mixing non-sensical comparison functions together is to the
+    programmer. If in doubt, you should rather use the {!Make}
+    functor for additional safety.
 
-   @author Xavier Leroy
    @author Nicolas Cannasse
    @author Markus Mottl
    @author David Rajchenbach-Teller
 *)
 
 type 'a t
-  (** The type of sets. *)
+(** The type of sets. *)
 
 include BatEnum.Enumerable with type 'a enumerable = 'a t
 include BatInterfaces.Mappable with type 'a mappable = 'a t
@@ -306,12 +297,12 @@ val empty: 'a t
 val create : ('a -> 'a -> int) -> 'a t
   (** Creates a new empty set, using the provided function for key comparison.*)
 
+val is_empty: 'a t -> bool
+  (** Test whether a set is empty or not. *)
+
 val singleton : ?cmp:('a -> 'a -> int) -> 'a -> 'a t
 (** Creates a new set with the single given element in it. *)
 
-val is_empty: 'a t -> bool
-  (** Test whether a set is empty or not. *)
-  
 val mem: 'a -> 'a t -> bool
   (** [mem x s] tests whether [x] belongs to the set [s]. *)
   
@@ -322,75 +313,6 @@ val add: 'a -> 'a t -> 'a t
 val remove: 'a -> 'a t -> 'a t
   (** [remove x s] returns a set containing all elements of [s],
       except [x]. If [x] was not in [s], [s] is returned unchanged. *)
-  
-  
-val iter: ('a -> unit) -> 'a t -> unit
-  (** [iter f s] applies [f] in turn to all elements of [s].
-      The elements of [s] are presented to [f] in increasing order
-      with respect to the ordering over the type of the elements. *)
-
-val map: ('a -> 'b) -> 'a t -> 'b t
-  (** [map f x] creates a new set with elements [f a0],
-      [f a1]... [f an], where [a1], ..., [an] are the
-      values contained in [x]*)
-  
-val filter: ('a -> bool) -> 'a t -> 'a t
-  (** [filter p s] returns the set of all elements in [s]
-      that satisfy predicate [p]. *)
-  
-val filter_map: ('a -> 'b option) -> 'a t -> 'b t
-  (** [filter_map f m] combines the features of [filter] and
-      [map].  It calls calls [f a0], [f a1], [f an] where [a0..an]
-      are the elements of [m] and returns the set of pairs [bi]
-      such as [f ai = Some bi] (when [f] returns [None], the
-      corresponding element of [m] is discarded). *)
-  
-val fold: ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
-  (** [fold f s a] computes [(f xN ... (f x2 (f x1 a))...)],
-      where [x1 ... xN] are the elements of [s], in increasing order. *)
-  
-val exists: ('a -> bool) -> 'a t -> bool
-  (** [exists p s] checks if at least one element of
-      the set satisfies the predicate [p]. *)
-  
-  
-val cardinal: 'a t -> int
-  (** Return the number of elements of a set. *)
-
-
-val choose : 'a t -> 'a
-  (** returns one binding of the given map, deterministically.  Raises
-      [Invalid_argument] if given an empty set. *)
-
-val min_elt : 'a t -> 'a
-  (** returns the binding with the smallest key. Raises
-      [Invalid_argument] if given an empty set. *)
-
-val max_elt : 'a t -> 'a
-  (** returns the binding with the largest key. Raises
-      [Invalid_argument] if given an empty set.*)
-  
-val enum: 'a t -> 'a BatEnum.t
-  (** Return an enumeration of all elements of the given set.
-      The returned enumeration is sorted in increasing order with respect
-      to the ordering of this set.*)
-  
-val of_enum: 'a BatEnum.t -> 'a t
-
-val for_all : ('a -> bool) -> 'a t -> bool
-(** Returns whether the given predicate applies to all elements in the set *)
-
-
-val partition : ('a -> bool) -> 'a t -> 'a t * 'a t
-  (** returns two disjoint subsets, those that satisfy the given
-      predicate and those that don't *)
-
-val filter : ('a -> bool) -> 'a t -> 'a t
-  (** returns the subset of items satisfying the given predicate *)
-
-val pop : 'a t -> 'a * 'a t
-  (** returns one element of the set and the set without that element.
-      Raises [Not_found] if given an empty set *)
 
 val union: 'a t -> 'a t -> 'a t
   (** [union s t] returns the union of [s] and [t] - the set containing
@@ -398,10 +320,105 @@ val union: 'a t -> 'a t -> 'a t
       comparison function.  The current implementation works better for
       small [s]. *)
 
+(* Set.Make uses intersect *)
+val intersect: 'a t -> 'a t -> 'a t
+(** [intersect s t] returns a new set of those elements that are in
+    both [s] and [t].  The returned set uses [s]'s comparison function. *)
+
 val diff: 'a t -> 'a t -> 'a t
   (** [diff s t] returns the set of all elements in [s] but not in
       [t]. The returned set uses [s]'s comparison function.*)
 
+(* No compare nor equal; they don't make sense if the comparison
+   functions of the two sets may be different, but apparently this
+   wasn't a problem for 'subset'? *)
+
+val subset: 'a t -> 'a t -> bool
+(** [subset a b] returns true if [a] is a subset of [b]. O(|a|). *)
+
+val iter: ('a -> unit) -> 'a t -> unit
+  (** [iter f s] applies [f] in turn to all elements of [s].
+      The elements of [s] are presented to [f] in increasing order
+      with respect to the ordering over the type of the elements. *)
+
+(* under-specified; either give a 'b comparison,
+   or keep ('a -> 'a) (preferred choice) *)
+val map: ('a -> 'b) -> 'a t -> 'b t
+  (** [map f x] creates a new set with elements [f a0],
+      [f a1]... [f aN], where [a0], [a1], ..., [aN] are the
+      values contained in [x]*)
+
+val filter: ('a -> bool) -> 'a t -> 'a t
+  (** [filter p s] returns the set of all elements in [s]
+      that satisfy predicate [p]. *)
+  
+(* as under-specified as 'map' *)
+val filter_map: ('a -> 'b option) -> 'a t -> 'b t
+  (** [filter_map f m] combines the features of [filter] and
+      [map].  It calls calls [f a0], [f a1], [f aN] where [a0,a1..an]
+      are the elements of [m] and returns the set of pairs [bi]
+      such as [f ai = Some bi] (when [f] returns [None], the
+      corresponding element of [m] is discarded). *)
+  
+val fold: ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
+  (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
+      where [x0,x1..xN] are the elements of [s], in increasing order. *)
+  
+val exists: ('a -> bool) -> 'a t -> bool
+  (** [exists p s] checks if at least one element of
+      the set satisfies the predicate [p]. *)
+
+val for_all : ('a -> bool) -> 'a t -> bool
+(** Returns whether the given predicate applies to all elements in the set *)
+
+val partition : ('a -> bool) -> 'a t -> 'a t * 'a t
+  (** returns two disjoint subsets, those that satisfy the given
+      predicate and those that don't *)
+
+val split : 'a -> 'a t -> 'a t * bool * 'a t
+  (** [split x s] returns a triple [(l, present, r)], where
+      [l] is the set of elements of [s] that are
+      strictly less than [x];
+      [r] is the set of elements of [s] that are
+      strictly greater than [x];
+      [present] is [false] if [s] contains no element equal to [x],
+      or [true] if [s] contains an element equal to [x]. *)
+
+val cardinal: 'a t -> int
+  (** Return the number of elements of a set. *)
+
+(* 'elements' missing *)
+
+val min_elt : 'a t -> 'a
+  (** returns the smallest element of the set. Raises
+      [Invalid_argument] if given an empty set. *)
+
+val max_elt : 'a t -> 'a
+  (** returns the largest element of the set. Raises
+      [Invalid_argument] if given an empty set.*)
+
+val choose : 'a t -> 'a
+  (** returns an arbitrary (but deterministic) element of the given set.
+      Raises [Invalid_argument] if given an empty set. *)
+
+val pop : 'a t -> 'a * 'a t
+  (** returns one element of the set and the set without that element.
+      Raises [Not_found] if given an empty set *)
+
+val enum: 'a t -> 'a BatEnum.t
+  (** Return an enumeration of all elements of the given set.
+      The returned enumeration is sorted in increasing order with respect
+      to the ordering of this set.*)
+  
+val of_enum: 'a BatEnum.t -> 'a t
+
+val of_enum_cmp: cmp:('a -> 'a -> int) -> 'a BatEnum.t -> 'a t
+
+(* of_list has no Set.Make counterpart *)
+val of_list: 'a list -> 'a t
+(** builds a set from the given list, using the default comparison
+    function *)
+
 (** {6 Boilerplate code}*)
 
 
